/**
 * This class generates IDL for WebFlow applications from XML.
 */
package WebFlow;

import com.ibm.xml.*;
import com.ibm.xml.parsers.*;
import java.io.*;
import org.w3c.dom.*;
//import org.apache.xerces.parsers.*;

public class IDLGenerator {
    DOMParser parser;
    Document doc;
    String xmlfilename;
    public IDLGenerator(String xmlfilename) {
	parser=initializeParser(xmlfilename);
	doc=initializeDoc(parser);
    }

    public DOMParser initializeParser(String xmlfilename) {
	DOMParser parser=null;
	try {
	    parser=new DOMParser();
	    parser.parse(xmlfilename);
	    System.out.println("Document is valid and well-formed");
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	    System.exit(1);
	}
	return parser;
    }	    
	
    /**
     * Transform the xml document into an idl document.
     */ 
    public void createIDL(String idlfile) {
	//This is the first node of the document;
	PrintWriter out=initIDLFile(idlfile);
	printHeader(out);
	int closecount=0,indentcnt=0;
	String indentstr="   ";

	//Set up first module line.
	Node firstNode=doc.getFirstChild();
	Node nextNode=firstNode.getNextSibling();

	String attstring=((Element)nextNode).getAttribute("name");
	out.println(firstNode.getNodeName()+" "+attstring+" {");
	closecount++;
	indentcnt++;

	//Set up second "module" line.
	NodeList nl=nextNode.getChildNodes();
	String nodename="",attnames="";
	Node saveNode=null;
	for(int i=0;i<nl.getLength();i++) {
	    if(nl.item(i).getNodeType()==Node.ELEMENT_NODE) {
		nodename=(nl.item(i)).getNodeName();
		attnames=((Element)nl.item(i)).getAttribute("name");
		saveNode=nl.item(i);
	    }
	}
	out.println(indentstr+nodename+" "+attnames+ " {");
	closecount++;
	indentcnt++;

	//Set up "interface" line.
	nl=saveNode.getChildNodes();
	for(int i=0;i<nl.getLength();i++) {
	    if(nl.item(i).getNodeType()==Node.ELEMENT_NODE) {
		nodename=(nl.item(i)).getNodeName();
		attnames=((Element)nl.item(i)).getAttribute("name");
		attnames+=":"+((Element)nl.item(i)).getAttribute("extends");
		saveNode=nl.item(i);
	    }
	}
	out.println(indentstr+indentstr+nodename+" "+attnames+ " {");
	closecount++;
	
	//Set up the method declarations
	nl=saveNode.getChildNodes();
	String methodname="",returntype="",arg="";
	NodeList nl2;
	for(int i=0;i<nl.getLength();i++) {
	    String attlist="";
	    if(nl.item(i).getNodeType()==Node.ELEMENT_NODE) {
		returntype=((Element)nl.item(i)).getAttribute("return");
		methodname=((Element)nl.item(i)).getAttribute("name");
		nl2=nl.item(i).getChildNodes();
		for(int j=0;j<nl2.getLength();j++) {
		    if(nl2.item(j).getNodeType()==Node.ELEMENT_NODE) {
			attlist+="in ";
			attlist+=((Element)nl2.item(j)).getAttribute("in")+" ";
			attlist+=nl2.item(j).getFirstChild().getNodeValue();
			if(j!=nl2.getLength()-2) attlist+=",";
		    }
		}
		out.println(indentstr+indentstr+indentstr+returntype+" "+methodname+"("+attlist+");");
		returntype="";
		methodname="";
		attlist="";
	    }
	}
	


	//Close all of the brackets we opened.
	for(int i=0;i<closecount;i++){
	    String indenter="";
	    for(int j=0;j<indentcnt;j++) {
		indenter+=indentstr;
	    }
	    indentcnt--;
	    out.println(indenter+"};");
	}
    }

    /**
     * Creates the output stream for the new IDL file.
     */
    public PrintWriter initIDLFile(String idlfile) {
	PrintWriter out=null;
	try {
	    out=new PrintWriter(new FileWriter(idlfile),true);
	}
	catch(Exception ex) {
	    ex.printStackTrace();
	    System.exit(1);
	}
	return out;
    }

    public void printHeader(PrintWriter out) {
	out.println("#ifndef _WEBFLOW_");
	out.println("#include \"../BC.idl\"");
	out.println("#endif");
    }

    public Document initializeDoc(DOMParser parser) {
	Document doc=parser.getDocument();
	return doc;
    }
    
    /**
     * A main() method for testing.
     */
    public static void main(String args[]) {
	DOMParser parser;
	Document doc;
	if(args.length != 1) {
	    System.out.println("Need XML file as arg");
	    System.exit(1);
	}
	
	IDLGenerator idlgen=new IDLGenerator(args[0]);
	idlgen.createIDL("test.idl");
    } 
}
