/**
 * Program: AuthenticateBean.java
 * Comments: Gets form data from the Login.jsp page, runs the kinit-batch
 * program to generate the ticket on the server. 
 */

package extensions.gem;

import java.util.*;
import java.io.*;
import javax.servlet.http.*;
import java.net.*;
import java.security.*;

public class AuthenticateBean {

    //Class variables
    HttpServletRequest request;
    private String remoteAddress;   //Remote client's IP address
    private Cookie authCookie;      //The cookie that contains the hashed 
    private File credfile=null;     //The kerberos cache file
    private MessageDigest digest;
    private String digestString;
    private String batchPath="";
    private String principal="";

    //Bean's no-argument constructor
    public AuthenticateBean() {
    }
    
    /**
     * Returns the parameter value from the HTTP request.
     */
    public String getParam(HttpServletRequest request,String paramName) {
	return request.getParameter(paramName);
    }
    
    /**
     * Get the user name (minus the realm info).
     */
    public String getUserName() {
	if(principal.indexOf("@")>0){
	    String userName=principal.substring(0,principal.indexOf("@"));
	    return userName;
	}
	else 
	    return principal;
    }

    /**
     * Get the location of the credentials file, needed by KRB5CCNAME.
     */
    public String getKrb5ccname() { 
	if (principal!=null) 
	    return "/tmp/krb5cc_"+principal;
	else 
	    return null;
    }

    /**
     * Test stuff out.
     */
    public void requestTest(HttpServletRequest request) {
	System.out.println(request.getRemoteHost());
    }
    
    /**
     * Run the kinit script and assign the member data.
     * The cookie should only be sent to the original domain 
     * (i.e. claw.arl.hpc.mil) and everything under the /GOW/CSM
     * directory will see the cookie.  These settings can be
     * overridden with setDomain() and setPath().  Security should 
     * be set to true.  
     */
    public HttpServletResponse setUpLogin(HttpServletRequest request,
					  HttpServletResponse response) {
	runLogin(request);
	if(credfile!=null && credfile.exists()) {
	    digest=digestFile();
	    digestString=createDigestString();
	    remoteAddress=request.getRemoteHost();
	    authCookie=setUpCookie(authCookie);
	    response.addCookie(authCookie);
	}
	return response;
    }
    
    private Cookie setUpCookie(Cookie authCookie) {
	int maxLife=8*60*60;  //Lives eight hours
	authCookie=new Cookie("authData",URLEncoder.encode(digestString));
	authCookie.setMaxAge(maxLife);
	authCookie.setSecure(false);     //must go over SSL?
	return authCookie;
    }

    private MessageDigest digestFile() {
	MessageDigest md=null;
	try {
	    md=MessageDigest.getInstance("MD5");
	    md.reset();
	    FileInputStream in=new FileInputStream(credfile);
	    int ch;
	    while((ch=in.read()) != -1){
		md.update((byte)ch);
	    }
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	return md;
    }

    private String createDigestString() {
	byte[] hash=digest.digest();
	String dstring="";
	for(int i=0;i<hash.length;i++) {
	    int v=hash[i] & 0xFF;
	    if(v<16) dstring+="0";
	    dstring+=Integer.toString(v,16).toUpperCase()+" ";
	}
	return dstring;
    }

    /**
     * Runs the kinit-batch program to generate a kerberos ticket on the
     * host.  Assigns the value to the credentials file.
     */
    public void runLogin(HttpServletRequest request) {
	//I will assume for now that kinit-batch is in the same directory as
	//AuthenticateBean.java

	principal=request.getParameter("UserName");
	String password=request.getParameter("Password");
	String passcode=request.getParameter("Passcode");

	String[] command=new String[4];
	command[0]=batchPath;
	command[1]=password;
	command[2]=passcode;
	command[3]=principal;
	try {
	    credfile=new File("/tmp/krb5cc_"+principal);
	    runCommand(command);
	}
	catch(Exception ex) { 
	    ex.printStackTrace();
	}
    }

    public boolean runCommand(String[] command) {
	boolean success=false;
	Runtime rt = Runtime.getRuntime();
        String commandOutput=""; 
        try {
	    
            Process p = rt.exec(command);
            BufferedReader in=
		new BufferedReader(new InputStreamReader(p.getInputStream()));
            
            BufferedReader err= 
              new BufferedReader(new InputStreamReader(p.getErrorStream()));

            String stdoutAndStderr = "STDOUT:\n";
            String line;
            while((line=in.readLine())!= null) {
              commandOutput+=line+"\n";
              stdoutAndStderr+=line;
            }
            System.out.println("Captured command output: "+commandOutput);
            in.close();
            stdoutAndStderr+="\nSTDERR:\n";
            while ((line=err.readLine())!=null) {
              stdoutAndStderr+=line;
            }
            err.close();
            p.waitFor();
            success=true;
        }
        catch (Exception eio) {
          System.err.println(eio.getMessage());
          eio.printStackTrace();
        }
	return success;
    }

    /**
     * Accessor methods for the batchPath property
     */
    public void setBatchPath(String batchPath) {
	this.batchPath=batchPath;
    }
    
    public String getBatchPath() {
	return batchPath;
    }


    /**
     * Check for the cookie's existence, compare its value to
     * the stored session value, and also check the remote client's
     * IP address.
     */
    public boolean validateRequest(HttpServletRequest request) {
	boolean validcookie=false;
	boolean validIP=false;
	String testVal="";
	Cookie[] cookies=request.getCookies();
	if(cookies!=null) {
	    for(int i=0;i<cookies.length;i++) {
		if(cookies[i].getName().equals("authData")) {
		    testVal=cookies[i].getValue();
		    break;
		}
	    }
	} 
	try {
	    testVal=URLDecoder.decode(testVal);
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	System.out.println(testVal);
	System.out.println(digestString);
	if(testVal.equals(digestString)){
	    validcookie=true;
	    System.out.println("Cookie valid "+validcookie);
	}
	
	if(request.getRemoteHost().equals(remoteAddress)){
	    validIP=true;
	    System.out.println("IP valid "+validIP);
	}
	return (validcookie&&validIP);  //both must be true
    }
    
    public void test() {
	System.out.println("Authenticate test");
    }
}


