/**
 * Program: ContextBean.java 
 * Comments: Implements WebFlowContext-style hashtable trees.
 */  

package extensions.gem;

import java.io.*;
import java.util.*;

public class ContextBean {

    Hashtable hashtable;
    String Name,parent;
    Hashtable children;
    String descDir;

    public ContextBean() {
	hashtable=new Hashtable();
	children=new Hashtable();
    }
    
    /**
     * For the given descriptor directory, restore recursively the 
     * existing contexts.
     */
    public void init(String name,String descDir) {
	this.descDir=descDir;
	Name=name;
    }

    /**
     * Create a new context.
     */
    public boolean createNewContext(String descDir,
				    String Name,
				    String parent){
	boolean created=false;
	this.Name=Name;
	hashtable.put("Children","null");
	hashtable.put("Descriptor","null");
	hashtable.put("Directory",descDir);
	hashtable.put("Name",Name);
	hashtable.put("Parent",parent);
	hashtable.put("CurrentChild","null");
	
	created=writeContextToFile(hashtable,descDir);

	return created;
    }
    
    /**
     * General methods for getting and setting some context value.
     * Use add/removeChildContext to manage children.
     */ 
    public String getContextValue(String name) {
	return (String) hashtable.get(name);
    }
    
    public void setContextValue(String name, String value) {
	hashtable.put(name,value);
	writeContextToFile(hashtable,descDir);
    }
    
    /**
     *
     */
    public ContextBean getContext() {
	return this;
    }

    /**
     * Add a child context
     */ 
    public void addChildContext(String childName){
	if(((String)hashtable.get("Children")).equals("null")) {
	    hashtable.put("Children",childName);
	}
	else if(((String)hashtable.get("Children")).trim().equals("")) {
	    hashtable.put("Children",childName);
	}
	else {
	    String oldlist=(String)hashtable.get("Children");
	    hashtable.put("Children",oldlist+"+"+childName);
	}
	ContextBean child=new ContextBean();
	child.init(childName,descDir+"/"+childName);
	child.createNewContext(descDir+"/"+childName,childName,Name);
	children.put(childName,child);
	writeContextToFile(hashtable,descDir);
	writeContextToFile(child.getContextHash(),descDir+"/"+childName);
    }

    /**
     * Remove a child context.  Note that we keep a running hashtable of 
     * the child names in the Hashtable children.
     */
    public boolean removeChildContext(String childName) {
	boolean success=false;
	String Children=(String)hashtable.get("Children");
	if(children.containsKey(childName))
	    children.remove(childName);
	if(Children!=null) {
	    StringTokenizer st=new StringTokenizer(Children,"+ ");
	    String newChildren="";
	    hashtable.put("Children","null");
	    int count=0;
	    boolean iflag=false;
	    while(st.hasMoreTokens()) {
		String item=st.nextToken();
		if(item.trim().equals(childName)) {
		    deleteChildDir(descDir+"/"+childName);
		    iflag=true;
		}
		else {
		    newChildren+="+"+item.trim();
		}
	    }
	    if(iflag)
		hashtable.put("Children",newChildren);
	    if(newChildren.startsWith("+"))
	       newChildren=newChildren.substring(1);
	    success=true;
	}
	writeContextToFile(hashtable,descDir);
	return true;
    }    
    
    /**
     * This method deletes all files and subdirectories and finally
     * the directory itself.  Note that applying File.delete() 
     * to a directory only works if the directory is empty.
     */ 
    public boolean deleteChildDir(String childDir) {
	boolean success=false;
        try {
            File file=new File(childDir);
            String[] files=file.list(); 
            for(int i= 0;i<files.length;i++) {
                File aFile = new File(childDir+File.separator+files[i]);
                if(aFile.isDirectory()) 
                    deleteChildDir(childDir+File.separator+ files[i]);
                else {
                    aFile.delete(); // delete file
		}
            }
            success=file.delete(); // delete directory
        } 
	catch (Exception ex) {
            ex.printStackTrace();
        }
	return success;
    }

    /**
     * Writes the context hash to the specified directory.
     */ 
    public boolean writeContextToFile(Hashtable ht,String descDir) {
	boolean success=false;
	Enumeration en=ht.keys();
	FileWriter fw;
	String name,value;

	try {  
	    File dir=new File(descDir);
	    if(!dir.exists())
		dir.mkdir();
	    fw=new FileWriter(descDir+"/ContextData.txt");
	    while(en.hasMoreElements()) {
		name=(String)en.nextElement();
		value=(String)ht.get(name);
		fw.write(name+" "+value+"\n");
		fw.flush();
	    }
	    fw.close();
	    success=true;
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	return success;
    }
    
    /**
     * Restores the context from the specified directory.  Recursively
     * adds all children.
     */ 
    public void restoreContext(String descDir) {
	try {
	    File file=new File(descDir+"/ContextData.txt");
	    if(file.exists()) {
		BufferedReader in=
		    new BufferedReader(new FileReader(file));
		String s=in.readLine();
		String aname,avalue;
		while(s != null){
		    StringTokenizer st=new StringTokenizer(s);
		    aname=st.nextToken();
		    avalue=st.nextToken();
		    hashtable.put(aname,avalue);
		    s=in.readLine();
		}
		in.close();
		
		//Now restore all children.
		String Children=(String)hashtable.get("Children");
		if(!Children.trim().equals("null")) {
		    StringTokenizer st1=new StringTokenizer(Children,"+");
		    while(st1.hasMoreTokens()) {
			String child=st1.nextToken();
			restoreChildContext(child);
		    }
		}
	    }
	    else {
		//First time logging in, so make the context.
		createNewContext(descDir,Name,"None");
	    }
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
    } 

    public void restoreChildContext(String childName) {
	String childDescDir=descDir+File.separator+childName;
	ContextBean child=new ContextBean();
	child.init(childName,childDescDir);
	child.restoreContext(childDescDir);
	children.put(childName,child);
    }
    
    public Hashtable getContextHash() {
	return hashtable;
    }

    /**
     * Returns the child contexts.
     */
    public Hashtable getChildren() {
	return children;
    }

    public ContextBean getChild(String childName) {
	return (ContextBean)children.get(childName);
    }


    /**
     * Simple accessor for context name.
     */
    public String getName() {
	return Name;
    }
    
}

