/**
 * Program:ContextManagerBean.java
 * Implements ContextManager's interface as a bean, not a webflow module. 
 * The ContextBean userContext is the user's base context.
 */

package extensions.gem;

import java.util.*;
import java.io.*;

public class ContextManagerBean {
    ContextBean userContext,currentProblem,currentSession;
    String userName,currentProblemName=null,currentSessionName=null;

    //Empty constructor.
    public ContextManagerBean() {
    }

    /**
     * Restores the user's contexts.
     */ 
    public void init(String userName,String descDir) {
	this.userName=userName;
	userContext=new ContextBean();
	userContext.init(userName,descDir);
	userContext.restoreContext(descDir);
	currentProblemName=userContext.getContextValue("CurrentChild");
    }

    /**
     * Sets the problem and session context data.
     */ 
    public void selectSession(String problemName,String sessionName) {
	currentProblem=userContext.getChild(problemName);
	currentProblemName=problemName;
	currentSession=currentProblem.getChild(sessionName);
	currentSessionName=sessionName;
	
	userContext.setContextValue("CurrentChild",problemName);
	currentProblem.setContextValue("CurrentChild",sessionName);
    }
    
    /**
     *
     */
    public String getCurrentSessionProperty(String propname){
	return currentSession.getContextValue(propname);	
    }
    
    public String getCurrentProblemProperty(String propname) {
	return currentProblem.getContextValue(propname);
    }

    public String getUserProperty(String propname) {
	return userContext.getContextValue(propname);
    }

    /**
     *
     */ 
    public void setCurrentSessionProperty(String propName,
					  String propValue ) {
	currentSession.setContextValue(propName,propValue);
    }

    public void setCurrentProblemProperty(String propName,
					  String propValue) {
	currentProblem.setContextValue(propName,propValue);
    }

    public void setUserProperty(String propName,String propValue) {
	userContext.setContextValue(propName,propValue);
    }


    /**
     *
     */ 
    public void addSession(String newSessionName) {
	currentProblem.addChildContext(newSessionName);
	currentProblem.setContextValue("CurrentChild",newSessionName);
	this.selectSession(currentProblemName,newSessionName);
    }
    
    /**
     *
     */
    public void removeCurrentSession() {
	currentProblem.removeChildContext(currentSessionName);
	currentProblem.setContextValue("CurrentChild","null");
    }

    public void removeProblem(String probName) {
	userContext.removeChildContext(probName);
	if(userContext.getContextValue("CurrentChild").trim().equals(probName)) userContext.setContextValue("CurrentChild","null");

    }

    public void addNewProblem(String problemName){
	userContext.addChildContext(problemName);
	currentProblemName=problemName;
	
	userContext.setContextValue("CurrentChild",problemName);
    }

    public void addNewSession(String problemName,String sessionName) {
	currentProblem=userContext.getChild(problemName);
	currentProblemName=problemName;
	addSession(sessionName);
    }

    public String[] listProblems() {
	return listChildNames(userContext);
    }
    
    public String[] listProblemSessions(String probName) {
	ContextBean problem=userContext.getChild(probName);
	return listChildNames(problem);
    }
    
    private String[] listChildNames(ContextBean context) {
	Hashtable list=context.getChildren();
	String[] names;
	if(list.size()>0) {
	    names=new String[list.size()];
	    Enumeration en=list.keys();
	    int increment=0;
	    while(en.hasMoreElements()){
		names[increment]=(String)en.nextElement();
		increment++;
	    }
	}
	else {
	    names=new String[1];
	    names[0]="null";
	}
	return names;
    }
    
    public void test() {
	System.out.println("ContextManger is running");
    }
    
} 
