/**
 * Program Name: createLSF.java
 * Comments: This class is a child of the createScript abstract parent.  
 * Creates a LSF script.
 * 
 * Note that the ContextManager object cm and the string scriptString are
 * protected data inherited from the parent.
 *
 * I am assuming the following naming convention for the context data:
 *
 *   shell: the unix shell to use.
 *   queue: the particular queue (default, challenge, and so on).
 *   Name: the name to give the job.
 *   NNodes: the number of nodes to request.
 *   walltime: the maximum time per node.
 *   Memory: the desired amount of memory.
 * Note that all of these variables are not required.
 */

package extensions.gem;
import WebFlow.*;
import WebFlow.ContextManager.*;
import java.io.*;
import java.util.*;

public class createLSF extends createScript {
    Hashtable ht;

    public createLSF(moduleServerBean msb,String userName){
	super(msb,userName);
    }

    public createLSF(ContextManager cm,String userName){
	super(cm,userName);
    }

    public createLSF(ContextManagerBean cm,String userName){
	super(cm,userName);
    }

    /**
     * This method concatenates the LSF-specific header directives
     * to the string scriptString, inherited from the parent.
     * It probably wouldn't hurt at some point to associate the 
     * param names with their flags (shell and -S, for example).
     */
    public void createHeader() {
	scriptString="#!/bin/csh -f\n";
	checkHash();
    }
    
    public void createFlagTable() {
	ht=new Hashtable();
	ht.put("Name","#BSUB -J ");
	ht.put("Memory","#BSUB -M ");
	ht.put("NNodes","#BSUB -n ");
	ht.put("walltime","#BSUB -W ");
	return;
    }

    private void checkHash() {
	if(my_cm!=null) {
	    String name,val;
	    Enumeration en=ht.keys();
	    while(en.hasMoreElements()){
		name=(String)en.nextElement();
		if(name.equals("Name")) {
		    scriptString+=(String)ht.get(name)
			+my_cm.getCurrentProblemProperty(name)
			+"."+my_cm.getCurrentSessionProperty(name)+"\n";
		}
		else if(!(my_cm.getCurrentSessionProperty(name)==null)){
		    scriptString+=(String)ht.get(name)
			+my_cm.getCurrentSessionProperty(name)+"\n";
		}    
	    }
	}
	else 
	    System.err.println("Some error in create LSF");
    }
}





