/**
 * Program Name: parseXMLBean.java
 * Comments: Several convenience methods for manipulating the 
 * ApplDesc.xml file
 * 
 * This Bean may be converted into a webflow module later.
 *
 * Note the package name.  XSIL looks for extensions in a directory called
 * "extensions" in the classpath.  In the ApplDesc.xml file, we have the
 * line
 *       <XSIL Name="Application Descriptor" Type="gem.parseXMLDesc">
 * so we must put parseXMLDesc.class in the extensions/gem/ 
 * directory somewhere in the classpath (such as WEB-INF/classes).
 */

package extensions.gem;

import java.util.*;
import org.escience.XSIL.*;
import java.io.*;

public class parseXMLBean {

    static parseXMLList parsexmllist;
    static XSIL root;
    static parseXMLDesc parsexmldesc;
    static parseXMLHost parsexmlhost;

    public parseXMLBean(){
    }

    public static void init(String xmlFileName) {
	boolean ss = true;
	root = new XSIL(xmlFileName,ss);
	XSIL child;

	for(int ichild=0;ichild<root.getChildCount();ichild++) {
            child = root.getChild(ichild);
	    parsexmllist = (parseXMLList)child;
        }
    }
    
    /**
     * This method should return the names of all of the applications.
     */

    public Vector getApplications() {
	Vector appList = parsexmllist.getParseXMLappListNames();
	return appList;
    }

    /**
     * This method returns the availabe hosts for a particular application.
     * application. Must provide application id.
     */

    public Vector getHostListNames(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp = null;

	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}

	Vector hostname = parsexmldesc.getHostListNames();
	return hostname;
    }

    /**
     * This method returns a hash table of all the top level tags for
     * a particular application and their values.
     * @param appIdVal the name of the application
     * @return Hashtable a hashtable of params and values.
     */
    public Hashtable getCodeTagList(String appIdVal) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	Hashtable appParamList= new Hashtable();
	String tmp=null;
	
	//Loop through application name list and find the one requested.
	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	appParamList = parsexmldesc.getCodeTagList();
	return appParamList;
    }

    /**
     * This method returns the string value assocatied with a 
     * particular tag name for a code.
     * @param appIdVal the name of the application
     * @param paramName the name of the parameter
     * @return String the value of the specified parameter 
     */
    public String getCodeTagValue(String appIdVal, String paramName) {
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String appParamVal=null;
	String tmp=null;

	//Loop through application name list and find the one requested.
	for(int i = 0; i < appListNames.size(); i++) {
	    tmp = (String)appListNames.elementAt(i);
	    if(tmp.equals(appIdVal)) { 
		parsexmldesc = (parseXMLDesc)appList.elementAt(i);
		break;
	    }
	}
	return parsexmldesc.getCodeTagValue(paramName);
    }

    /**
     * This method returns a vector of all the top level tags for
     * a particular machine and application.
     * @param appIdVal the name of the application
     * @param machineIdVal the name of the host machine
     * @return Hashtable a hash of the params and their values.
     */
    public Hashtable getMachineTagList(String appIdVal, String machineIdVal) {
	 Vector appList = parsexmllist.getParseXMLappList();
	 Vector appListNames = parsexmllist.getParseXMLappListNames();
	 String tmp1 = null,tmp2=null;
	 String tagVal=null;
	 parseXMLDesc pxml=null;

	 //Find the matching application from the list.
	 for(int j=0;j<appListNames.size();j++) {
	     tmp1=(String)appListNames.elementAt(j);
	     if(tmp1.equals(appIdVal)) {
		 pxml=(parseXMLDesc)appList.elementAt(j);
		 break;
	     }
	 }
	 
	 //For the given application, find the named host.
	 for(int i = 0; i < pxml.getHostListNames().size(); i++) {
	     tmp2 = (String)pxml.getHostListNames().elementAt(i);
	     if(tmp2.equals(machineIdVal)) {
		 parsexmlhost = (parseXMLHost)pxml.getHostList().elementAt(i);
		 break;
	     }
	 }
	 return parsexmlhost.getParamHash();
    }

    /**
     * This method returns the string value for the particular
     * named parameter for a code on a particular host machine.
     * a particular machine and application.
     * @param appIdVal the name of the application
     * @param machineIdVal the name of the host machine
     * @param paramName the name of the parameter we want evaluated
     * @return Vector a vector of string names.
     */
    public String getMachineTagValue(String appIdVal, 
				    String machineIdVal,
				    String paramName) {
	 Vector appList = parsexmllist.getParseXMLappList();
	 Vector appListNames = parsexmllist.getParseXMLappListNames();
	 String tmp1 = null,tmp2=null;
	 String tagVal=null;
	 parseXMLDesc pxml=null;

	 //Find the matching application.
	 for(int j=0;j<appListNames.size();j++) {
	     tmp1=(String)appListNames.elementAt(j);
	     if(tmp1.equals(appIdVal)) {
		 pxml=(parseXMLDesc)appList.elementAt(j);
		 break;
	     }
	 }
	 
	 //For the given application, find the named host.
	 for(int i = 0; i < pxml.getHostListNames().size(); i++) {
	     tmp2 = (String)pxml.getHostListNames().elementAt(i);
	     if(tmp2.equals(machineIdVal)) {
		 parsexmlhost = (parseXMLHost)pxml.getHostList().elementAt(i);
		 break;
	     }
	 }
	 
	 return parsexmlhost.getTagValue(paramName);
    }
    
    public Hashtable getEnvironHash(String appId,String machineId){
	//Find the matching application and host from the lists.
	Vector appList = parsexmllist.getParseXMLappList();
	Vector appListNames = parsexmllist.getParseXMLappListNames();
	String tmp1 = null,tmp2=null;
	String tagVal=null;
	parseXMLDesc pxml=null;
	Hashtable returnHash=null;
	
	for(int j=0;j<appListNames.size();j++) {
	    tmp1=(String)appListNames.elementAt(j);
	    if(tmp1.equals(appId)) {
		System.out.println("Found matching app tag");
		pxml=(parseXMLDesc)appList.elementAt(j);
		break;
	    }
	}
	
	for(int i = 0; i < pxml.getHostListNames().size(); i++) {
	    tmp2 = (String)pxml.getHostListNames().elementAt(i);
	    if(tmp2.equals(machineId)) {
		System.out.println("Found matching machine tag");
		parsexmlhost = (parseXMLHost)pxml.getHostList().elementAt(i);
		break;
	    }
	}
	returnHash=parsexmlhost.getEnvironHash();
	return returnHash;
    }    
    
    public void test() {
	System.out.println("Parser is up");
    }
}    






