/**
 * @author Marlon Pierce
 *
 * This file extracts parameter values of the XSIL Host tag, which
 * describes a particular machine.
 */ 
package extensions.gem;
import org.escience.XSIL.*;
import java.util.*;

public class parseXMLHost extends XSIL {

    public String hostname;
    public String classpath,ldlibrary,ansysProduct,lmLicense;
    public Hashtable environhash;
    public Hashtable paramHash=new Hashtable();    //a hashtable to hold param names/values.
    
    boolean instantiated = false,constructed=false;
    
    public void construct(){	
	environhash=new Hashtable();
	for(int ichild=0; ichild < getChildCount(); ichild++){
            XSIL x = getChild(ichild);
            if(x instanceof Param){
		Param p = (Param)x;
		paramHash.put(p.getName(),p.getText());
	    }
	    //The table should be only one row deep.
	    if(x instanceof Table) {
		Table t=(Table)x;
		for(int i=0; i< t.getNColumn();i++) {
		    Column c = t.getColumn(i);
		    String envName=c.getName();
		    for(int j=0; j < t.getNRow(); j++) {
			String envValue = c.getPrimArray().getString(j);
			//A hack because XSIL can't handle a one
			//column, one row table
			if(!envName.trim().equals("EMPTY")) 
			    environhash.put(envName,envValue);
		    }
		}
	    }
	}
	constructed=true;
    }
    
    // This function does the heavy lifting of reading arrays and so on
    // It is separate from the construction because it is not called 
    // unless necessary.

    public void instantiate(){
	construct();
	instantiated=true;
    }
    
    /**
     * Returns the hashtable of params and their values.
     */
    public Hashtable getParamHash() {
	if(!constructed) construct();
	return paramHash;
    }
    
    /**
     * Returns the value for the specified parameter.
     */
    public String getTagValue(String paramName) {
	if(!constructed) construct();
	if(paramHash.containsKey(paramName)) {
	    return (String)paramHash.get(paramName);
	}
	else {
	    System.err.println("Parameter not found!");
	    return null;
	}
    }
    
    public Hashtable getEnvironHash() {
	return environhash;
    }

    public String getHost() { 
	if(!constructed) construct();
	return (String)paramHash.get("HostName");
    }



    /**
     * The following methods are obsolete.
     */ 
    
    public String getLMLicense() {
	return lmLicense;
    }
    public String getAnsysProduct() {
	return ansysProduct;
    }
    public String getLDLibrary() {
	return ldlibrary;
    }
    public String getClasspath() {
	return classpath;
    }
}
